@@ -15,8 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- *  $Id$
  */
 
 #include "config.h"
@@ -656,7 +654,7 @@ scm_dealable_set_sensitive (SCM in_state)
 }
 
 static SCM
-scm_get_feature_word ()
+scm_get_feature_word (void)
 {
   AisleriotGame *game = app_game;
 
@@ -917,7 +915,7 @@ scm_set_timeout (SCM new)
 }
 
 static SCM
-scm_get_timeout ()
+scm_get_timeout (void)
 {
   AisleriotGame *game = app_game;
 
@@ -984,7 +982,7 @@ scm_delayed_call (SCM callback)
 }
 
 static void
-cscm_init ()
+cscm_init (void)
 {
   /* Enable useful debugging options. */
   SCM_DEVAL_P = 1;
@@ -1035,7 +1033,6 @@ update_game_dealable (AisleriotGame *game)
                             cscmi_call_lambda, &data,
                             cscmi_catch_handler, NULL);
 
-  g_print ("dealable returned %d\n", SCM_NFALSEP (data.retval));
   set_game_dealable (game, SCM_NFALSEP (data.retval));
 }
 
@@ -1437,7 +1434,7 @@ aisleriot_game_set_paused (AisleriotGame *game,
   if (paused == game->paused)
     return;
 
-  paused = game->paused;
+  game->paused = paused;
 
   /* (Re)store the start time */
   game->start_time = time (NULL) - game->start_time;
@@ -1602,6 +1599,9 @@ aisleriot_game_new_game (AisleriotGame *game,
    * But treat a restart as part of the same game. Eventually either
    * the player will win or lose and then it gets counted.
    */
+  /* FIXMEchpe: this allows cheating the statistics by doing
+   * Restart, then New Game.
+   */
   if (game->state >= GAME_RUNNING &&
       (!seed || *seed != game->seed)) {
     update_statistics (game);
@@ -1729,7 +1729,6 @@ aisleriot_game_drag_valid (AisleriotGame *game,
                            guint n_cards)
 {
   CallData data = CALL_DATA_INIT;
-  SCM retval;
 
   data.lambda = game->button_pressed_lambda;
   data.n_args = 2;
@@ -1739,10 +1738,9 @@ aisleriot_game_drag_valid (AisleriotGame *game,
   scm_internal_stack_catch (SCM_BOOL_T,
                             cscmi_call_lambda, &data,
                             cscmi_catch_handler, NULL);
-  retval = data.retval;
   scm_gc_unprotect_object (data.arg2);
 
-  return SCM_NFALSEP (retval);
+  return SCM_NFALSEP (data.retval);
 }
 
 /**
