@@ -11,13 +11,7 @@ __all__ = ['GtkUI']
 import os
 import sys
 import time
-import gettext
-
-try:
-    import pygtk
-    pygtk.require('2.0')
-except ImportError, err:
-    print ("PyGTK not found. Please make sure it is installed properly and referenced in your PYTHONPATH environment variable.")
+from gettext import gettext as _
 
 import gobject
 import gtk
@@ -41,183 +35,16 @@ import glchess.config
 import glchess.ui
 import glchess.chess.board
 import dialogs
+import log
 import chessview
+import network
+
+# Mark all windows with our icon
+gtk.window_set_default_icon_name(ICON_NAME)
 
 def loadGladeFile(name, root = None):
     return gtk.glade.XML(os.path.join(GLADE_DIR, name), root, domain = DOMAIN)
 
-class GtkGameNotebook(gtk.Notebook):
-    """
-    """
-    
-    glConfig          = None
-
-    defaultView       = None
-    views             = None
-    viewsByWidget     = None
-    
-    def __init__(self, ui):
-        """
-        """
-        self.ui = ui
-        self.views = []
-        self.viewsByWidget = {}
-        
-        gtk.Notebook.__init__(self)
-        self.set_show_border(False)
-        
-        # Make the tabs scrollable so the area is not resized
-        self.set_scrollable(True)
-        
-        # Configure openGL
-        try:
-            gtk.gdkgl
-        except AttributeError:
-            self.glConfig = None
-        else:
-            display_mode = (gtk.gdkgl.MODE_RGB | gtk.gdkgl.MODE_DEPTH | gtk.gdkgl.MODE_DOUBLE)
-            try:
-                self.glConfig = gtk.gdkgl.Config(mode = display_mode)
-            except gtk.gdkgl.NoMatches:
-                display_mode &= ~gtk.gdkgl.MODE_DOUBLE
-                self.glConfig = gtk.gdkgl.Config(mode = display_mode)
-            
-        self.set_show_tabs(False)
-        
-    def setDefault(self, feedback):
-        """
-        """
-        assert(self.defaultView is None)
-        self.defaultView = chessview.GtkView(self.ui, '', feedback)
-        page = self.append_page(self.defaultView.widget)
-        self.set_current_page(page)
-        
-        self.__updateTabVisibleState()
-        
-        return self.defaultView
-
-    def addView(self, title, feedback):
-        """
-        """
-        view = chessview.GtkView(self.ui, title, feedback, moveFormat = self.ui.moveFormat)
-        self.views.append(view)
-        self.viewsByWidget[view.widget] = view
-        page = self.append_page(view.widget)
-        self.set_tab_label_text(view.widget, title)
-        self.set_current_page(page)
-        
-        self.__updateTabVisibleState()
-        
-        return view
-    
-    def getView(self):
-        """Get the currently selected view.
-        
-        Returns the view (GtkView) on this page or None if no view here.
-        """
-        # If splashscreen present then there is no view
-        if len(self.viewsByWidget) == 0:
-            return None
-        
-        num = self.get_current_page()
-        if num < 0:
-            return None
-        widget = self.get_nth_page(num)
-        return self.viewsByWidget[widget]
-    
-    def removeView(self, view):
-        """Remove a view from the notebook.
-        
-        'view' is the view to remove.
-        """
-        self.remove_page(self.page_num(view.widget))
-        self.views.remove(view)
-        self.viewsByWidget.pop(view.widget)
-        self.__updateTabVisibleState()
-
-    def __updateTabVisibleState(self):
-        """
-        """
-        # Only show tabs if there is more than one game
-        self.set_show_tabs(len(self.viewsByWidget) > 1)
-        
-        # Show/hide the default view
-        if len(self.viewsByWidget) == 0:
-            self.defaultView.widget.show()
-        else:
-            self.defaultView.widget.hide()
-        
-class AIWindow:
-    """
-    """
-    
-    notebook    = None
-    defaultPage = None
-    
-    # We keep track of the number of pages as there is a bug 
-    # in GtkNotebook (Gnome bug #331785).
-    pageCount   = 0
-    
-    def __init__(self, notebook):
-        """
-        """
-        self.notebook = notebook
-        self.defaultPage = notebook.get_nth_page(0)
-    
-    def addView(self, title, executable, description):
-        """
-        """
-        # Hide the default page
-        self.defaultPage.hide()
-        self.notebook.set_show_tabs(True)
-        
-        self.pageCount += 1
-        return AIView(self, title, executable, description)
-
-class AIView:
-    """
-    """
-    
-    __gui = None
-    
-    def __init__(self, window, title, executable, description):
-        """
-        """
-        self.window = window
-        self.__gui = loadGladeFile('ai.glade', 'ai_table')
-        self.__gui.get_widget('executable_label').set_text(executable)
-        self.__gui.get_widget('game_label').set_text(description)
-
-        # Add into the notebook
-        self.root = self.__gui.get_widget('ai_table')
-        notebook = window.notebook
-        notebook.append_page(self.root, gtk.Label(title))
-                
-        # Create styles for the buffer
-        buffer = self.__gui.get_widget('comms_textview').get_buffer()
-        buffer.create_tag('input', family='Monospace')
-        buffer.create_tag('output', family='Monospace', weight = pango.WEIGHT_BOLD)
-        buffer.create_tag('move', family='Monospace', foreground = 'blue')
-        buffer.create_tag('info', family='Monospace', foreground = 'green')
-        buffer.create_tag('error', family='Monospace', foreground = 'red')
-
-    def addText(self, text, style):
-        """FIXME: Define style
-        """
-        buffer = self.__gui.get_widget('comms_textview').get_buffer()
-        buffer.insert_with_tags_by_name(buffer.get_end_iter(), text, style)
-        
-    def close(self):
-        """
-        """
-        self.window.pageCount -= 1
-        self.window.notebook.remove_page(self.window.notebook.page_num(self.root))
-        
-        # Show the default page
-        if self.window.pageCount == 0:
-            self.window.defaultPage.show()
-            self.window.notebook.set_show_tabs(False)
-            
 class GLibTimer(glchess.ui.Timer):
     """
     """
@@ -335,9 +162,6 @@ class GtkUI(glchess.ui.UI):
     __lastTime         = None
     __animationTimer   = None
     
-    # The notebook containing games
-    notebook           = None
-
     # The Gtk+ list model of the available player types
     __playerModel      = None
 
@@ -359,8 +183,6 @@ class GtkUI(glchess.ui.UI):
 
     __attentionCounter = 0
 
-    moveFormat         = 'human'
-    
     whiteTimeString    = '∞'
     blackTimeString    = '∞'
     
@@ -369,6 +191,8 @@ class GtkUI(glchess.ui.UI):
     height             = None
     isFullscreen       = False
     isMaximised        = False
+    
+    view               = None
 
     def __init__(self, feedback):
         """Constructor for a GTK+ glChess GUI"""
@@ -380,17 +204,25 @@ class GtkUI(glchess.ui.UI):
         self.__networkGames = {}
         self.__saveGameDialogs = {}
         self.__joinGameDialogs = []
-
+        
         # Set the message panel to the tooltip style
         # (copied from Gedit)
+        # In Gtk+ 2.11+ (I think) tip_window is now private so skip if it's not there (bug #459740)
         tooltip = gtk.Tooltips()
         tooltip.force_window()
-        tooltip.tip_window.ensure_style()
-        self.tooltipStyle = tooltip.tip_window.get_style()
+        if hasattr(tooltip, 'tip_window') and tooltip.tip_window != None:
+            tooltip.tip_window.ensure_style()
+            self._tooltipStyle = tooltip.tip_window.get_style()
+        else:
+            self._tooltipStyle = None
+        self._tooltipWidgetsDrawn = {}
         
         self._gui = loadGladeFile('glchess.glade')
         self._gui.signal_autoconnect(self)
         
+        # Workaround as Glade 2 always overrides the system style for toolbars
+        self.__getWidget('toolbar').unset_style()
+        
         # Create mappings between the promotion radio buttons and the promotion types
         self.__promotionTypeByRadio = {}
         self.__promotionRadioByType = {}
@@ -407,29 +239,13 @@ class GtkUI(glchess.ui.UI):
             self.__boardViewTypeByRadio[widget] = name
             self.__boardViewRadioByType[name] = widget
 
-        # Make a notebook for the games
-        self.notebook = GtkGameNotebook(self)
-        self.notebook.connect_after('switch-page', self._on_view_changed)
-        self.__getWidget('game_viewport').add(self.notebook)
-        self.notebook.show_all()
-        
         # Create the model for the player types
-        self.__playerModel = gtk.ListStore(str, gtk.gdk.Pixbuf, str)
+        self.__playerModel = gtk.ListStore(str, str, str)
         iconTheme = gtk.icon_theme_get_default()
-        try:
-            icon = iconTheme.load_icon('stock_people', 24, gtk.ICON_LOOKUP_USE_BUILTIN)
-        except gobject.GError:
-            icon = None
         iter = self.__playerModel.append()
-        self.__playerModel.set(iter, 0, '', 1, icon, 2, gettext.gettext('Human'))
-        #try:
-        #    icon = iconTheme.load_icon(gtk.STOCK_NETWORK, 24, gtk.ICON_LOOKUP_USE_BUILTIN)
-        #except gobject.GError:
-        #    icon = None
-        #iter = self.__playerModel.append()
-        #self.__playerModel.set(iter, 0, None, 1, icon, 2, gettext.gettext('Network'))
+        self.__playerModel.set(iter, 0, '', 1, 'stock_person', 2, _('Human'))
         
-        self.__aiWindow = AIWindow(self._gui.get_widget('ai_notebook'))
+        self.__logWindow = log.LogWindow(self._gui.get_widget('log_notebook'))
 
         # Balance space on each side of the history combo
         group = gtk.SizeGroup(gtk.SIZE_GROUP_BOTH)
@@ -442,26 +258,54 @@ class GtkUI(glchess.ui.UI):
         combo.pack_start(cell, False)
         combo.add_attribute(cell, 'text', 2)
 
-        self.defaultViewController = self.notebook.setDefault(None)
-
         # Disble help support
         if haveGnomeSupport:
             self._gui.get_widget('menu_help').show()
         
-        self.defaultViewController.viewWidget.setRenderGL(self.__renderGL)
-
-        self.saveDialog = dialogs.SaveDialog(self)
+        self._updateViewButtons()
         
         # Watch for config changes
-        for key in ['show_toolbar', 'show_history', 'fullscreen', 'show_3d', 'show_move_hints', 'width', 'height', 'move_format', 'promotion_type', 'board_view']:
+        for key in ['show_toolbar', 'show_history', 'fullscreen',
+                    'show_3d', 'show_comments', 'show_numbering',
+                    'show_move_hints',
+                    'width', 'height',
+                    'move_format', 'promotion_type', 'board_view',
+                    'enable_networking']:
             glchess.config.watch(key, self.__applyConfig)
 
     # Public methods
     
+    def setTooltipStyle(self, widget):
+        """Set a widget to be in the tooltip style.
+        
+        'widget' is the widget to modify.
+        """
+        if self._tooltipStyle is None:
+            return
+        widget.set_style(self._tooltipStyle)
+        widget.connect("expose_event", self._on_tooltip_expose_event)
+        widget.queue_draw()
+
+    def _on_tooltip_expose_event(self, widget, event):
+        """Gtk+ callback"""
+        allocation = widget.allocation
+        widget.style.paint_flat_box(widget.window, gtk.STATE_NORMAL, gtk.SHADOW_OUT, None, widget, "tooltip",
+                                    allocation.x, allocation.y, allocation.width, allocation.height)
+                                    
+        # The first draw is corrupt for me so draw it twice.
+        # Bonus points to anyone who tracks down the problem and fixes it
+        if not self._tooltipWidgetsDrawn.has_key(widget):
+            self._tooltipWidgetsDrawn[widget] = True
+            widget.queue_draw()
+    
     def watchFileDescriptor(self, fd):
         """Extends ui.UI"""
         gobject.io_add_watch(fd, gobject.IO_IN | gobject.IO_HUP, self.__readData)
         
+    def writeFileDescriptor(self, fd):
+        """Extends ui.UI"""
+        gobject.io_add_watch(fd, gobject.IO_OUT, self.__writeData)
+        
     def addTimer(self, feedback, duration):
         """Extends ui.UI"""
         return GLibTimer(self, feedback, duration)
@@ -471,8 +315,13 @@ class GtkUI(glchess.ui.UI):
         return True
 
     def __readData(self, fd, condition):
+        #print (fd, condition)
         return self.feedback.onReadFileDescriptor(fd)
 
+    def __writeData(self, fd, condition):
+        #print (fd, condition)
+        return self.feedback.onWriteFileDescriptor(fd)
+
     def addAIEngine(self, name):
         """Register an AI engine.
         
@@ -480,32 +329,56 @@ class GtkUI(glchess.ui.UI):
         TODO: difficulty etc etc
         """
         iconTheme = gtk.icon_theme_get_default()
-        try:
-            icon = iconTheme.load_icon('stock_notebook', 24, gtk.ICON_LOOKUP_USE_BUILTIN)
-        except gobject.GError:
-            icon = None
         iter = self.__playerModel.append()
-        self.__playerModel.set(iter, 0, name, 1, icon, 2, name)
+        self.__playerModel.set(iter, 0, name, 1, 'stock_notebook', 2, name)
         
         # Get the human to play against this AI
         if self.__defaultBlackAI is None:
             self.__defaultBlackAI = name
-        
-    def setDefaultView(self, feedback):
-        """Extends ui.UI"""
-        self.defaultViewController.feedback = feedback
-        return self.defaultViewController
-        
-    def addView(self, title, feedback):
-        """Extends ui.UI"""
-        view = self.notebook.addView(title, feedback)
-        view.viewWidget.setRenderGL(self.__renderGL)
-        return view
 
-    def addAIWindow(self, title, executable, description):
+    def setView(self, title, feedback, isPlayable = True):
+        """Extends ui.UI"""
+        moveFormat = glchess.config.get('move_format')
+        showComments = glchess.config.get('show_comments')
+        self.view = chessview.GtkView(self, feedback, moveFormat = moveFormat, showComments = showComments)
+        self.view.setTitle(title)
+        self.view.isPlayable = isPlayable
+        self.view.viewWidget.setRenderGL(self.__renderGL)
+        viewport = self.__getWidget('game_viewport')
+        child = viewport.get_child()
+        if child is not None:
+            viewport.remove(child)
+        viewport.add(self.view.widget)
+
+        # Set toolbar/menu buttons to state for this game
+        self._updateViewButtons()
+        
+        # Update timers
+        if self.view is not None:
+            self.setTimers(self.view.whiteTime, self.view.blackTime)
+
+        return self.view
+
+    def updateTitle(self):
         """
         """
-        return self.__aiWindow.addView(title, executable, description)
+        # Set the window title to the name of the game
+        if self.view is not None and len(self.view.title) > 0:
+            if self.view.needsSaving:
+                # Translators: This is the window title when playing a game that needs saving
+                title = _('Chess - *%(game_name)s') % {'game_name': self.view.title}
+            else:
+                # Translators: This is the window title when playing a game that is saved
+                title = _('Chess - %(game_name)s') % {'game_name': self.view.title}
+        else:
+            # Translators: This is the window title when not playing a game
+            title = _('Chess')            
+        self._gui.get_widget('glchess_app').set_title(title)
+
+    def addLogWindow(self, title, executable, description):
+        """
+        """
+        return self.__logWindow.addView(title, executable, description)
     
     def setTimers(self, whiteTime, blackTime):
         """
@@ -534,7 +407,10 @@ class GtkUI(glchess.ui.UI):
         This method will not return.
         """        
         # Load configuration
-        for name in ['show_toolbar', 'show_history', 'show_3d', 'show_move_hints', 'move_format', 'promotion_type', 'board_view', 'maximised']:
+        for name in ['show_toolbar', 'show_history', 'show_3d',
+                     'show_comments', 'show_numbering', 'show_move_hints',
+                     'move_format', 'promotion_type', 'board_view', 'maximised',
+                     'enable_networking']:
             try:
                 value = glchess.config.get(name)
             except glchess.config.Error:
@@ -562,7 +438,7 @@ class GtkUI(glchess.ui.UI):
     def addNetworkDialog(self, feedback):
         """Extends glchess.ui.UI"""
         # Create the dialog
-        dialog = dialogs.GtkNetworkGameDialog(self, feedback, self.__playerModel)
+        dialog = network.GtkNetworkGameDialog(self, feedback, self.__playerModel)
         self.__joinGameDialogs.append(dialog)
         
         # Add the detected games into the dialog
@@ -586,7 +462,26 @@ class GtkUI(glchess.ui.UI):
         # Update the open dialogs
         for dialog in self.__joinGameDialogs:
             dialog.removeNetworkGame(game)
-        
+            
+    def requestSave(self, title):
+        """Extends glchess.ui.UI"""
+        dialog = gtk.MessageDialog(flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+                                   type = gtk.MESSAGE_WARNING,
+                                   message_format = title)
+        dialog.format_secondary_text("If you don't save the changes to this game will be permanently lost")
+        dialog.add_button(_('Close _without saving'), gtk.RESPONSE_OK)
+        dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT)
+        dialog.add_button(gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT)
+
+        response = dialog.run()
+        dialog.destroy()
+        if response == gtk.RESPONSE_ACCEPT:
+            return glchess.ui.SAVE_YES
+        elif response == gtk.RESPONSE_OK:
+            return glchess.ui.SAVE_NO
+        else:
+            return glchess.ui.SAVE_ABORT
+
     def close(self):
         """Extends glchess.ui.UI"""
         # Save the window size
@@ -625,14 +520,6 @@ class GtkUI(glchess.ui.UI):
             return error        
         self.__saveGameDialogs.pop(view)
 
-    def _removeView(self, view):
-        """Remove a view from the UI.
-        
-        'view' is the view to remove.
-        """
-        self.notebook.removeView(view)
-        self._updateViewButtons()
-
     # Private methods
 
     def __resize(self):
@@ -661,6 +548,16 @@ class GtkUI(glchess.ui.UI):
             else:
                 menu.set_active(False)
                 toolbar.hide()
+                
+        elif name == 'enable_networking':
+            menuItem = self.__getWidget('menu_play_online_item')
+            toolbarButton = self.__getWidget('play_online_button')
+            if value is True:
+                menuItem.show()
+                toolbarButton.show()
+            else:
+                menuItem.hide()
+                toolbarButton.hide()
             
         # Show/hide the history
         elif name == 'show_history':
@@ -692,27 +589,42 @@ class GtkUI(glchess.ui.UI):
         # Enable/disable OpenGL rendering
         elif name == 'show_3d':            
             if value and not chessview.haveGLSupport:
-                self._gui.get_widget('3d_support_dialog').show()
+                title = _('Unable to enable 3D mode')
+                errors = '\n'.join(chessview.openGLErrors)
+                description = _("""You are unable to play in 3D mode due to the following problems:
+%(errors)s
+
+Please contact your system administrator to resolve these problems, until then you will be able to play chess in 2D mode.""" % {'errors': errors})
+                dialog = gtk.MessageDialog(type = gtk.MESSAGE_WARNING, message_format = title)
+                dialog.format_secondary_text(description)
+                dialog.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
+                dialog.run()
+                dialog.destroy()
                 glchess.config.set('show_3d', False)
                 value = False
             self.__renderGL = value
             menuItem = self.__getWidget('menu_view_3d')
             menuItem.set_active(self.__renderGL)
-            self.notebook.defaultView.viewWidget.setRenderGL(self.__renderGL)
-            for view in self.notebook.views:
-                view.viewWidget.setRenderGL(self.__renderGL)
+            self.view.viewWidget.setRenderGL(self.__renderGL)
                 
+        elif name == 'show_comments':
+            menuItem = self.__getWidget('menu_view_comment')
+            menuItem.set_active(value)
+            self.view.setShowComments(value)
+
         elif name == 'show_move_hints':
             menuItem = self.__getWidget('menu_view_move_hints')
             menuItem.set_active(value)
-            for view in self.notebook.views:
-                view.feedback.showMoveHints(value)
-        
+            self.view.feedback.showMoveHints(value)
+
+        elif name == 'show_numbering':
+            menuItem = self.__getWidget('menu_view_numbering')
+            menuItem.set_active(value)
+            self.view.feedback.showBoardNumbering(value)
+
         elif name == 'move_format':
             self._gui.get_widget('menu_movef_%s' % value).set_active(True)
-            self.moveFormat = value
-            for view in self.notebook.views:
-                view.setMoveFormat(value)
+            self.view.setMoveFormat(value)
                 
         elif name == 'promotion_type':
             try:
@@ -762,7 +674,7 @@ class GtkUI(glchess.ui.UI):
         widget = self._gui.get_widget(name)
         assert(widget is not None), 'Unable to find widget: %s' % name
         return widget
-    
+
     def _on_white_time_paint(self, widget, event):
         """Gtk+ callback"""
         self.__drawTime(self.whiteTimeString, widget, (0.0, 0.0, 0.0), (1.0, 1.0, 1.0))
@@ -818,6 +730,22 @@ class GtkUI(glchess.ui.UI):
             value = False
         glchess.config.set('show_3d', value)
 
+    def _on_menu_view_comment_clicked(self, widget):
+        """Gtk+ callback"""
+        if widget.get_active():
+            value = True
+        else:
+            value = False
+        glchess.config.set('show_comments', value)
+
+    def _on_menu_view_numbering_clicked(self, widget):
+        """Gtk+ callback"""
+        if widget.get_active():
+            value = True
+        else:
+            value = False
+        glchess.config.set('show_numbering', value)       
+
     def _on_toggle_move_hints_clicked(self, widget):
         """Gtk+ callback"""
         if widget.get_active():
@@ -826,9 +754,9 @@ class GtkUI(glchess.ui.UI):
             value = False
         glchess.config.set('show_move_hints', value)       
 
-    def _on_show_ai_stats_clicked(self, widget):
+    def _on_show_logs_clicked(self, widget):
         """Gtk+ callback"""
-        window = self._gui.get_widget('ai_window')
+        window = self._gui.get_widget('log_window')
         if widget.get_active():
             window.show()
         else:
@@ -844,15 +772,18 @@ class GtkUI(glchess.ui.UI):
         # Get the move number
         moveNumber = model.get_value(iter, 1)
         
-        string = 'Show move number: ' + str(moveNumber)
         if moveNumber == len(model) - 1:
-            string += ' (latest)'
             moveNumber = -1
-        
-        view = self.notebook.getView()
-        if view is not None:
-            view._setMoveNumber(moveNumber)
-            
+
+        # Disable buttons when at the end
+        haveMoves = len(model) > 1
+        for widget in ('first_move_button', 'prev_move_button'):
+            self.__getWidget(widget).set_sensitive(haveMoves and moveNumber != 0)
+        for widget in ('last_move_button', 'next_move_button'):
+            self.__getWidget(widget).set_sensitive(haveMoves and moveNumber != -1)
+
+        self.view._setMoveNumber(moveNumber)
+
     def _on_menu_movef_human_activate(self, widget):
         """Gtk+ callback"""
         glchess.config.set('move_format', 'human')
@@ -922,44 +853,28 @@ class GtkUI(glchess.ui.UI):
         """Gtk+ callback"""
         self.__selectMoveNumber(-1)
 
-    def _on_view_changed(self, widget, page, pageNum):
-        """Gtk+ callback"""
-        # Set the window title to the name of the game
-        view = self.notebook.getView()
-        title = gettext.gettext('Chess')
-        if view is not None:
-            title += " - %s" % view.title
-        self._gui.get_widget('glchess_app').set_title(title)
-
-        # Set toolbar/menu buttons to state for this game
-        self._updateViewButtons()
-        
-        # Update timers
-        if view is not None:
-            self.setTimers(view.whiteTime, view.blackTime)
-    
     def _updateViewButtons(self):
         """
         """
-        view = self.notebook.getView()
-        enableWidgets = (view is not None) and view.isActive
-        self.__getWidget('end_game_button').set_sensitive(enableWidgets)
-        self.__getWidget('save_game_button').set_sensitive(enableWidgets)
-        self.__getWidget('menu_save_item').set_sensitive(enableWidgets)
-        self.__getWidget('menu_save_as_item').set_sensitive(enableWidgets)
-        self.__getWidget('menu_end_game_item').set_sensitive(enableWidgets)
+        enable = self.view is not None and self.view.isPlayable
+        for widget in ('save_game_button', 'menu_save_item', 'menu_save_as_item'):
+            self.__getWidget(widget).set_sensitive(enable)
 
         combo = self.__getWidget('history_combo')
-        if view is None:
+        if self.view is None:
             if combo.get_model() != None:
                 combo.set_model(None)
         else:
-            (model, selected) = view._getModel()
+            (model, selected) = self.view._getModel()
             combo.set_model(model)
             if selected < 0:
                 selected = len(model) + selected
             combo.set_active(selected)
-        self.__getWidget('navigation_box').set_sensitive(enableWidgets)
+        self.__getWidget('navigation_box').set_sensitive(enable)
+        
+        enable = enable and self.view.gameResult is None
+        for widget in ('menu_resign', 'resign_button', 'menu_claim_draw'):
+            self.__getWidget(widget).set_sensitive(enable)
 
     def _on_new_game_button_clicked(self, widget):
         """Gtk+ callback"""
@@ -975,25 +890,23 @@ class GtkUI(glchess.ui.UI):
         
     def _on_save_game_button_clicked(self, widget):
         """Gtk+ callback"""
-        view = self.notebook.getView()
-        
-        if view.feedback.getFileName() is not None:
-            view.feedback.save()
-        elif not self.__saveGameDialogs.has_key(view):
-            self.__saveGameDialogs[view] = dialogs.GtkSaveGameDialog(self, view)
+        if self.view.feedback.getFileName() is not None:
+            self.view.feedback.save()
+        elif not self.__saveGameDialogs.has_key(self.view):
+            self.__saveGameDialogs[self.view] = dialogs.GtkSaveGameDialog(self, self.view)
             
     def _on_save_as_game_button_clicked(self, widget):
         """Gtk+ callback"""
-        view = self.notebook.getView()
-        if not self.__saveGameDialogs.has_key(view):
-            self.__saveGameDialogs[view] = dialogs.GtkSaveGameDialog(self, view, view.feedback.getFileName())
+        if not self.__saveGameDialogs.has_key(self.view):
+            self.__saveGameDialogs[self.view] = dialogs.GtkSaveGameDialog(self, self.view, self.view.feedback.getFileName())
 
-    def _on_end_game_button_clicked(self, widget):
+    def _on_resign_clicked(self, widget):
         """Gtk+ callback"""
-        view = self.notebook.getView()
-        assert(view is not None)
-        if view.feedback is not None:
-            view.feedback.close()
+        self.view.feedback.resign()
+
+    def _on_claim_draw_clicked(self, widget):
+        """Gtk+ callback"""
+        self.view.feedback.claimDraw()
 
     def _on_help_clicked(self, widget):
         """Gtk+ callback"""
@@ -1001,7 +914,7 @@ class GtkUI(glchess.ui.UI):
             gnome.help_display('glchess')
         except gobject.GError, e:
             # TODO: This should be a pop-up dialog
-            print gettext.gettext('Unable to display help: %s') % str(e)
+            print _('Unable to display help: %s') % str(e)
 
     def _on_view_fullscreen_clicked(self, widget):
         """Gtk+ callback"""
@@ -1037,13 +950,10 @@ class GtkUI(glchess.ui.UI):
         dialog.set_comments(DESCRIPTION)
         dialog.set_authors(AUTHORS)
         dialog.set_artists(ARTISTS)
-        string = ''
-        for t in TRANSLATORS:
-            string += t + '\n'
-        dialog.set_translator_credits(string[:-1])
+        dialog.set_translator_credits(_("translator-credits"))
         dialog.set_website(WEBSITE)
         dialog.set_website_label(WEBSITE_LABEL)
-        dialog.set_logo_icon_name('glchess')
+        dialog.set_logo_icon_name(ICON_NAME)
         dialog.connect('response', self._on_glchess_about_dialog_close)
         dialog.show()
         
@@ -1053,31 +963,12 @@ class GtkUI(glchess.ui.UI):
         self.__aboutDialog = None
         return False
         
-    def _on_ai_window_delete_event(self, widget, event):
+    def _on_log_window_delete_event(self, widget, event):
         """Gtk+ callback"""
-        self._gui.get_widget('menu_view_ai').set_active(False)
+        self._gui.get_widget('menu_view_logs').set_active(False)
         
         # Stop the event - the window will be closed by the menu event
         return True
-   
-    def _on_save_dialog_response(self, widget, responseId):
-        """Gtk+ callback"""
-        self.saveDialog.setVisible(False)
-        
-        if responseId != gtk.RESPONSE_OK:
-            return
-        
-        # Save the requested games
-        for (view, save, _) in self.saveDialog.model:
-            if save:
-                view.feedback.save()
-
-        self.feedback.onQuit()
-        
-    def _on_save_dialog_delete(self, widget, event):
-        """Gtk+ callback"""
-        # Leave it to use to hide the dialog
-        return True
     
     def _on_resize(self, widget, event):
         """Gtk+ callback"""
@@ -1103,25 +994,12 @@ class GtkUI(glchess.ui.UI):
 
     def _on_close_window(self, widget, event):
         """Gtk+ callback"""
-        self._quit()
+        self.feedback.onQuit()
         return True
         
     def _on_menu_quit(self, widget):
         """Gtk+ callback"""
-        self._quit()
-        
-    def _quit(self):        
-        # Check if any views need saving
-        viewsToSave = []
-        for view in self.notebook.views:
-            if view.feedback.needsSaving():
-                viewsToSave.append(view)
-
-        if len(viewsToSave) == 0:
-            self.feedback.onQuit()
-        else:
-            self.saveDialog.setViews(viewsToSave)
-            self.saveDialog.setVisible(True)
+        self.feedback.onQuit()
 
 if __name__ == '__main__':
     ui = GtkUI()
