@@ -1,7 +1,7 @@
 /*
  * games-sound.c: common sound player for gnome-games 
  *
- * Copyright (C) 2007 Andreas Røsdal
+ * Copyright © 2007 Andreas Røsdal
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,14 +21,19 @@
 
 #include <config.h>
 
-#include <gnome.h>
 #include <glib.h>
+#include <glib/gi18n.h>
+
+#ifdef HAVE_GSTREAMER
 #include <gst/gst.h>
+#endif /* HAVE_GSTREAMER */
 
 #include "games-sound.h"
 
+#ifdef HAVE_GSTREAMER
+
 static GstElement *pipeline;
-static gboolean sound_enabled = TRUE;
+static gboolean sound_enabled = FALSE;
 static gboolean sound_init = FALSE;
 static GThreadPool *threads;
 
@@ -97,23 +102,33 @@ games_sound_init (void)
 
 }
 
+#endif /* HAVE_GSTREAMER */
+
 /**
- * games_sound_get_option_group:
+ * games_sound_add_option_group:
+ * @context: a #GOptionContext
  *
- * Returns: a new #GOptionGroup containing the GStreamer options
+ * Adds the GStreamer option group to @context.
  */
-GOptionGroup *
-games_sound_get_option_group (void)
+void
+games_sound_add_option_group (GOptionContext *context)
 {
-  return gst_init_get_option_group ();
+#ifdef HAVE_GSTREAMER
+  g_option_context_add_group (context, gst_init_get_option_group ());
+#endif /* HAVE_GSTREAMER */
 }
 
-/* Plays a sound with the given filename using GStreamer. The sound file is stored in 
+/**
+ * games_sound_play:
+ * @filename: the sound file to player
+ * 
+ * Plays a sound with the given filename using GStreamer. The sound file is stored in
  * the SOUNDDIR directory in .ogg format. Sound is played in a separate thread.
  */
 void
 games_sound_play (const gchar * filename)
 {
+#ifdef HAVE_GSTREAMER
   GError *err = NULL;
 
   if (!sound_enabled)
@@ -122,19 +137,49 @@ games_sound_play (const gchar * filename)
     games_sound_init ();
 
   g_thread_pool_push (threads, (gchar *) filename, &err);
-
+#endif /* HAVE_GSTREAMER */
 }
 
-/* Enables or disables sound support. */
+/**
+ * games_sound_enable:
+ * @enabled:
+ *
+ * Enables or disables sound support.
+ */
 void
-games_sound_enable (gboolean status)
+games_sound_enable (gboolean enabled)
 {
-  sound_enabled = status;
+#ifdef HAVE_GSTREAMER
+  sound_enabled = enabled;
+#endif /* HAVE_GSTREAMER */
 }
 
-/* Determines if sound support is enabled. */
+/**
+ * games_sound_is_enabled:
+ *
+ * Returns: %TRUE iff sound support is enabled.
+ */
 gboolean
 games_sound_is_enabled (void)
 {
+#ifdef HAVE_GSTREAMER
   return sound_enabled;
+#else
+  return FALSE;
+#endif /* HAVE_GSTREAMER */
+}
+
+/**
+ * games_sound_is_available:
+ *
+ * Returns: whether sound is available
+ */
+gboolean
+games_sound_is_available (void)
+{
+#ifdef HAVE_GSTREAMER
+  return TRUE;
+#else
+  return FALSE;
+#endif /* HAVE_GSTREAMER */
 }
